/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package view;
import model.*;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.*;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;
import javax.swing.border.EtchedBorder;

import controller.Dispatcher;

/**
 * <p>Title: Application</p>
 * <p>Description: Class managing the runtime application instance</p>
 * <p>Copyright: Copyright (c) 2003 **fill me** </p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.11 $
 */
public class Application extends JFrame {
  
  	private Window window;
  	private Menu menu;
  	private Document doc;
	
	// TODO SB To be corrected.... Not a good OO practice
	public JPanel parametersPanel;
	
  	static Application theInstance;
  	
	/**
	 * Default constructor.
	 */
  	public Application() {}
  	
	/**
	 * Get the active document.
	 * @return active document instance
	 */
	public Document getActiveDocument(){
		return doc;
	}  	

	/**
	 * Get the current instance of the application.
	 * @return current instance of the application
	 */
  	static public Application getInstance(){
		return theInstance;
  	}
  	
  	/**
  	 * Launches the application
  	 */
  	private void launchApplication(){
  		doc = new Document();
		Dispatcher dispatcher = new Dispatcher();
    	window = new Window(dispatcher);
	    menu = new Menu();
	    JPanel objectsPanel = window;

		ButtonsPanel buttonsPanel = new ButtonsPanel(dispatcher);
		buttonsPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		parametersPanel = new JPanel();
		parametersPanel.setLayout(new CardLayout());

		// TODO EP We should remove the old ways of dealing with the user interface, thus
		// keeping only the else section.				
		int ui = 4; // 1 = color parameters, 2 = fill parameters, 3 = filtering parameters, 4 = card layout
		if (ui == 1) {
			parametersPanel = new ColorParameterPanel(doc);
		} else if (ui == 2) {
			parametersPanel = new FillPanel(dispatcher.getTransformersIndex().getTheFiller());
		} else if (ui == 3){
			parametersPanel = new FilterKernelPanel(dispatcher.getTransformersIndex());
		} else {
			parametersPanel.add(new ColorParameterPanel(doc), "ColorParameterPanel");
			parametersPanel.add(new FillPanel(dispatcher.getTransformersIndex().getTheFiller()), "FillPanel");
			parametersPanel.add(new FilterKernelPanel(dispatcher.getTransformersIndex()), "FilterKernelPanel");
			parametersPanel.add(new CurvesPanel(dispatcher.getTransformersIndex()), "CurvesPanel");
			parametersPanel.add(new TranslationTransformation(dispatcher.getTransformersIndex()), "TranslationTransformation");
			parametersPanel.add(new ScallingTransformation(dispatcher.getTransformersIndex()), "ScallingTransformation");
			parametersPanel.add(new ShearingTransformation(dispatcher.getTransformersIndex()), "ShearingTransformation");
			parametersPanel.add(new RotationTransformation(dispatcher.getTransformersIndex()), "RotationTransformation");
		}
		
		getContentPane().add(parametersPanel, BorderLayout.NORTH);
		getContentPane().add(buttonsPanel, BorderLayout.WEST);
	    getContentPane().add(window, BorderLayout.CENTER);
	    
	    setJMenuBar(menu);
	    setSize(800, 600);
	    //pack();
	    //show();
	    setVisible(true);
    	
    	addWindowListener(new WindowAdapter() { 
	    	public void windowClosing(WindowEvent e) {
	        	System.exit(0);
	        }
	    });
  	}
  	
	public void colorSelectedObjects(Pixel p) {
		if (p == null)
			return;
			
		// TODO This application logic should go in the controller or the model.
		List list = theInstance.getActiveDocument().getSelectedObjects();
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shape s = (Shape)i.next();
			if (s instanceof ColoredShape) {
				ColoredShape cs = (ColoredShape)s;
				cs.setColor(p);
			}
		}
	}

  	/**
  	 * MAIN
  	 * @param args main's parameters
  	 */
  	public static void main(String[] args) {
    	theInstance = new Application();
    	theInstance.launchApplication();
  	}
}